/*
 * Copyright 2016 University of Chicago
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * @file module_base.c
 * Implementation of the ceph/s3 DSI
 */

#include "globus_gridftp_server_s3_base.h"


/* Configuration options */
/* Configuration options */
char                                   *globus_i_s3_default_hostname;
char                                   *globus_i_s3_mime_type_file;
bool                                    globus_i_s3_https = true;
bool                                    globus_i_s3_use_virtual_host = false;
bool                                    globus_i_s3_verify_peer = true;
int                                     globus_i_s3_max_retries = 1;
int                                     globus_i_s3_retry_delay_ms = 100;

globus_hashtable_t                      globus_i_s3_mime_extensions;

const char *                            globus_i_s3_debug_level_names[] =
{
    [GLOBUS_S3_DEBUG_TRACE]           = "TRACE",
    [GLOBUS_S3_DEBUG_INFO]            = "INFO",
    [GLOBUS_S3_DEBUG_DEBUG]           = "DEBUG",
    [GLOBUS_S3_DEBUG_WARNING]         = "WARN",
    [GLOBUS_S3_DEBUG_ERROR]           = "ERROR"
};

GlobusDebugDefine(GLOBUS_S3);

static
int
globus_l_s3_base_activate(void)
{
    S3Status                            status;
    globus_result_t                     result = GLOBUS_SUCCESS;

    GlobusDebugInit(GLOBUS_S3, TRACE INFO DEBUG WARNING ERROR);

    result = globus_i_s3_mime_types_parse(globus_i_s3_mime_type_file);
    if (result != GLOBUS_SUCCESS)
    {
        goto mime_parse_fail;
    }

    GlobusS3Debug("initializing to use host: %s\n",
                  globus_i_s3_default_hostname);

    status = S3_initialize(
            NULL,
            ((globus_i_s3_https && globus_i_s3_verify_peer) ? S3_INIT_VERIFY_PEER : 0) | S3_INIT_ALL,
            globus_i_s3_default_hostname);

    if (status != S3StatusOK)
    {
        GlobusS3Error("S3_initialize returns: %d", status);
        goto s3_init_fail;
    }

    GlobusS3ExitResult(result);
    return result;

s3_init_fail:
    globus_hashtable_destroy_all(&globus_i_s3_mime_extensions, free);
mime_parse_fail:
    GlobusS3ExitResult(result ? result : GLOBUS_FAILURE);
    return result ? result : GLOBUS_FAILURE;
}
/* globus_l_s3_base_activate() */

static
int
globus_l_s3_base_deactivate(void)
{
    globus_hashtable_destroy_all(&globus_i_s3_mime_extensions, free);
    S3_deinitialize();
    GlobusDebugDestroy(GLOBUS_S3);

    return 0;
}
/* globus_l_s3_base_deactivate() */

globus_module_descriptor_t
globus_i_s3_base_module =
{
    .module_name = "globus_gridftp_server_s3_base",
    .activation_func = globus_l_s3_base_activate,
    .deactivation_func = globus_l_s3_base_deactivate
};
// vim: ft=c:
